@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -38,16 +38,16 @@
 #include "iconv-detect.h"
 #else /* use old-style detection */
 #if defined (__aix__) || defined (__irix__) || defined (__sun__)
-#define ICONV_ISO_D_FORMAT "ISO%d-%d"
+#define ICONV_ISO_INT_FORMAT "ISO%u-%u"
 /* this one is for charsets like ISO-2022-JP, for which at least
    Solaris wants a - after the ISO */
-#define ICONV_ISO_S_FORMAT "ISO-%d-%s"
+#define ICONV_ISO_STR_FORMAT "ISO-%u-%s"
 #elif defined (__hpux__)
-#define ICONV_ISO_D_FORMAT "iso%d%d"
-#define ICONV_ISO_S_FORMAT "iso%d%s"
+#define ICONV_ISO_INT_FORMAT "iso%u%u"
+#define ICONV_ISO_STR_FORMAT "iso%u%s"
 #else
-#define ICONV_ISO_D_FORMAT "iso-%d-%d"
-#define ICONV_ISO_S_FORMAT "iso-%d-%s"
+#define ICONV_ISO_INT_FORMAT "iso-%u-%u"
+#define ICONV_ISO_STR_FORMAT "iso-%u-%s"
 #endif /* __aix__, __irix__, __sun__ */
 #define ICONV_10646 "iso-10646"
 #endif /* USE_ICONV_DETECT */
@@ -331,13 +331,13 @@ g_mime_locale_language (void)
  * @charset. Currently only handles CJK and Russian/Ukranian
  * charset->lang mapping. Everything else will return %NULL.
  *
- * Returns a language code that is specific to @charset, or NULL on
+ * Returns a language code that is specific to @charset, or %NULL on
  * fail.
  **/
 const char *
 g_mime_charset_language (const char *charset)
 {
-	int i;
+	guint i;
 	
 	if (!charset)
 		return NULL;
@@ -422,11 +422,11 @@ g_mime_charset_iconv_name (const char *charset)
 					iconv_name = g_strdup ("IBM-921");
 				else
 #endif /* __aix__ */
-					iconv_name = g_strdup_printf (ICONV_ISO_D_FORMAT,
+					iconv_name = g_strdup_printf (ICONV_ISO_INT_FORMAT,
 								      iso, codepage);
 			} else {
 				/* codepage is a string - probably iso-2022-jp or something */
-				iconv_name = g_strdup_printf (ICONV_ISO_S_FORMAT,
+				iconv_name = g_strdup_printf (ICONV_ISO_STR_FORMAT,
 							      iso, p);
 			}
 		} else {
@@ -511,7 +511,7 @@ g_mime_charset_canon_name (const char *charset)
 {
 	const char *ptr;
 	char *endptr;
-	int iso;
+	guint iso;
 	
 	if (!charset)
 		return NULL;
@@ -712,7 +712,7 @@ static const char *
 charset_best_mask (unsigned int mask)
 {
 	const char *lang;
-	int i;
+	guint i;
 	
 	for (i = 0; i < G_N_ELEMENTS (charinfo); i++) {
 		if (charinfo[i].bit & mask) {
