@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -36,6 +36,7 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <ctype.h>
+#include <poll.h>
 
 #include "gmime-gpg-context.h"
 #include "gmime-filter-charset.h"
@@ -230,9 +231,9 @@ struct _GpgCtx {
 	int passwd_fd;  /* only needed for sign/decrypt */
 	
 	/* status-fd buffer */
-	unsigned char *statusbuf;
-	unsigned char *statusptr;
-	unsigned int statusleft;
+	char *statusbuf;
+	char *statusptr;
+	guint statusleft;
 	
 	char *need_id;
 	char *passwd;
@@ -288,6 +289,7 @@ gpg_ctx_new (GMimeSession *session, const char *path)
 	gpg->seen_eof2 = FALSE;
 	gpg->pid = (pid_t) -1;
 	gpg->exit_status = 0;
+	gpg->flushed = FALSE;
 	gpg->exited = FALSE;
 	
 	gpg->path = g_strdup (path);
@@ -314,6 +316,7 @@ gpg_ctx_new (GMimeSession *session, const char *path)
 	gpg->need_id = NULL;
 	gpg->passwd = NULL;
 	
+	gpg->nodata = FALSE;
 	gpg->badsig = FALSE;
 	gpg->errsig = FALSE;
 	gpg->goodsig = FALSE;
@@ -427,11 +430,11 @@ gpg_ctx_get_diagnostics (struct _GpgCtx *gpg)
 {
 	if (!gpg->flushed) {
 		g_mime_stream_flush (gpg->diagnostics);
-		g_byte_array_append (gpg->diag, "", 1);
+		g_byte_array_append (gpg->diag, (unsigned char *) "", 1);
 		gpg->flushed = TRUE;
 	}
 	
-	return gpg->diag->data;
+	return (const char *) gpg->diag->data;
 }
 
 static void
@@ -728,10 +731,10 @@ gpg_ctx_op_start (struct _GpgCtx *gpg)
 	return -1;
 }
 
-static const char *
-next_token (const char *in, char **token)
+static char *
+next_token (char *in, char **token)
 {
-	const char *start, *inptr = in;
+	char *start, *inptr = in;
 	
 	while (*inptr == ' ')
 		inptr++;
@@ -755,10 +758,10 @@ next_token (const char *in, char **token)
 static int
 gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 {
-	register unsigned char *inptr;
-	const unsigned char *status;
 	size_t nread, nwritten;
+	register char *inptr;
 	GMimeSigner *signer;
+	char *status, *tmp;
 	int len;
 	
  parse:
@@ -778,9 +781,18 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 	d(printf ("status: %s\n", status));
 	
 	if (strncmp (status, "[GNUPG:] ", 9) != 0) {
+		if (!gpg->utf8)
+			tmp = g_locale_to_utf8 (status, -1, &nread, &nwritten, NULL);
+		else
+			tmp = status;
+		
 		g_set_error (err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR,
 			     _("Unexpected GnuPG status message encountered:\n\n%s"),
-			     status);
+			     tmp);
+		
+		if (!gpg->utf8)
+			g_free (tmp);
+		
 		return -1;
 	}
 	
@@ -898,9 +910,18 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 		}
 	} else if (!strncmp (status, "UNEXPECTED ", 11)) {
 		/* this is an error */
+		if (!gpg->utf8)
+			tmp = g_locale_to_utf8 (status + 11, -1, &nread, &nwritten, NULL);
+		else
+			tmp = status + 11;
+		
 		g_set_error (err, GMIME_ERROR, GMIME_ERROR_GENERAL,
 			     _("Unexpected response from GnuPG: %s"),
-			     status + 11);
+			     tmp);
+		
+		if (!gpg->utf8)
+			g_free (tmp);
+		
 		return -1;
 	} else if (!strncmp (status, "NODATA", 6)) {
 		/* this is an error */
@@ -909,7 +930,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 		diagnostics = gpg_ctx_get_diagnostics (gpg);
 		if (diagnostics && *diagnostics)
 			g_set_error (err, GMIME_ERROR, GMIME_ERROR_GENERAL,
-				     diagnostics);
+				     "%s", diagnostics);
 		else
 			g_set_error (err, GMIME_ERROR, GMIME_ERROR_GENERAL,
 				     _("No data provided"));
@@ -994,7 +1015,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 			} else if (!strncmp (status, "REVKEYSIG", 9)) {
 				gpg->signer->errors |= GMIME_SIGNER_ERROR_REVKEYSIG;
 			} else if (!strncmp (status, "VALIDSIG ", 9)) {
-				unsigned char *inend;
+				char *inend;
 				
 				gpg->validsig = TRUE;
 				status += 9;
@@ -1008,7 +1029,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 				status = next_token (status, NULL);
 				
 				/* the third token is the signature creation date (or 0 for unknown?) */
-				signer->sig_created = strtoul (status, (char **) &inend, 10);
+				signer->sig_created = strtoul (status, &inend, 10);
 				if (inend == status || *inend != ' ')
 					break;
 				
@@ -1099,62 +1120,61 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 	gpg->statusleft -= len;                                           \
 } G_STMT_END
 
+enum {
+	GPG_STDIN_FD,
+	GPG_STDOUT_FD,
+	GPG_STDERR_FD,
+	GPG_STATUS_FD,
+	GPG_PASSWD_FD,
+	GPG_N_FDS
+};
+
 static int
 gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 {
-	fd_set rdset, wrset, *wrsetp = NULL;
 	const char *diagnostics, *mode;
-	struct timeval timeout;
+	struct pollfd pfds[GPG_N_FDS];
 	int ready, save;
-	int maxfd = 0;
+	nfds_t n;
 	
- retry:
-	FD_ZERO (&rdset);
+	for (n = 0; n < GPG_N_FDS; n++) {
+		pfds[n].events = 0;
+		pfds[n].fd = -1;
+	}
 	
 	if (!gpg->seen_eof1) {
-		FD_SET (gpg->stdout_fd, &rdset);
-		maxfd = MAX (maxfd, gpg->stdout_fd);
+		pfds[GPG_STDOUT_FD].fd = gpg->stdout_fd;
+		pfds[GPG_STDOUT_FD].events = POLLIN;
 	}
 	
 	if (!gpg->seen_eof2) {
-		FD_SET (gpg->stderr_fd, &rdset);
-		maxfd = MAX (maxfd, gpg->stderr_fd);
+		pfds[GPG_STDERR_FD].fd = gpg->stderr_fd;
+		pfds[GPG_STDERR_FD].events = POLLIN;
 	}
 	
 	if (!gpg->complete) {
-		FD_SET (gpg->status_fd, &rdset);
-		maxfd = MAX (maxfd, gpg->status_fd);
+		pfds[GPG_STATUS_FD].fd = gpg->status_fd;
+		pfds[GPG_STATUS_FD].events = POLLIN;
 	}
 	
-	if (gpg->stdin_fd != -1 || gpg->passwd_fd != -1) {
-		FD_ZERO (&wrset);
-		if (gpg->istream && gpg->stdin_fd != -1) {
-			FD_SET (gpg->stdin_fd, &wrset);
-			maxfd = MAX (maxfd, gpg->stdin_fd);
-		}
-		if (gpg->passwd_fd != -1) {
-			FD_SET (gpg->passwd_fd, &wrset);
-			maxfd = MAX (maxfd, gpg->passwd_fd);
-		}
-		
-		wrsetp = &wrset;
-	}
+	pfds[GPG_STDIN_FD].fd = gpg->stdin_fd;
+	pfds[GPG_STDIN_FD].events = POLLOUT;
 	
-	g_assert (maxfd > 0);
+	pfds[GPG_PASSWD_FD].fd = gpg->passwd_fd;
+	pfds[GPG_PASSWD_FD].events = POLLOUT;
 	
-	timeout.tv_sec = 10; /* timeout in seconds */
-	timeout.tv_usec = 0;
+	do {
+		for (n = 0; n < GPG_N_FDS; n++)
+			pfds[n].revents = 0;
+		ready = poll (pfds, GPG_N_FDS, 10 * 1000);
+	} while (ready == -1 && errno == EINTR);
 	
-	if ((ready = select (maxfd + 1, &rdset, wrsetp, NULL, &timeout)) == 0)
+	if (ready == -1) {
+		d(printf ("poll() failed: %s\n", g_strerror (errno)));
+		goto exception;
+	} else if (ready == 0) {
+		/* timed out */
 		return 0;
-	
-	if (ready < 0) {
-		if (errno == EINTR)
-			goto retry;
-		
-		d(printf ("select() failed: %s\n", g_strerror (errno)));
-		
-		return -1;
 	}
 	
 	/* Test each and every file descriptor to see if it's 'ready',
@@ -1163,7 +1183,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 	   can to all of them. If one fails along the way, return
 	   -1. */
 	
-	if (FD_ISSET (gpg->status_fd, &rdset)) {
+	if (pfds[GPG_STATUS_FD].revents & (POLLIN | POLLHUP)) {
 		/* read the status message and decide what to do... */
 		char buffer[4096];
 		ssize_t nread;
@@ -1173,6 +1193,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 		do {
 			nread = read (gpg->status_fd, buffer, sizeof (buffer));
 		} while (nread == -1 && (errno == EINTR || errno == EAGAIN));
+		
 		if (nread == -1)
 			goto exception;
 		
@@ -1185,7 +1206,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 		}
 	}
 	
-	if (FD_ISSET (gpg->stdout_fd, &rdset) && gpg->ostream) {
+	if ((pfds[GPG_STDOUT_FD].revents & (POLLIN | POLLHUP)) && gpg->ostream) {
 		char buffer[4096];
 		ssize_t nread;
 		
@@ -1205,7 +1226,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 		}
 	}
 	
-	if (FD_ISSET (gpg->stderr_fd, &rdset)) {
+	if (pfds[GPG_STDERR_FD].revents & (POLLIN | POLLHUP)) {
 		char buffer[4096];
 		ssize_t nread;
 		
@@ -1224,7 +1245,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 		}
 	}
 	
-	if (wrsetp && gpg->passwd_fd != -1 && FD_ISSET (gpg->passwd_fd, &wrset) && gpg->need_passwd && gpg->send_passwd) {
+	if ((pfds[GPG_PASSWD_FD].revents & (POLLOUT | POLLHUP)) && gpg->need_passwd && gpg->send_passwd) {
 		ssize_t w, nwritten = 0;
 		size_t n;
 		
@@ -1252,7 +1273,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 		gpg->send_passwd = FALSE;
 	}
 	
-	if (gpg->istream && wrsetp && gpg->stdin_fd != -1 && FD_ISSET (gpg->stdin_fd, &wrset)) {
+	if ((pfds[GPG_STDIN_FD].revents & (POLLOUT | POLLHUP)) && gpg->istream) {
 		char buffer[4096];
 		ssize_t nread;
 		
@@ -1322,7 +1343,7 @@ gpg_ctx_op_step (struct _GpgCtx *gpg, GError **err)
 			     diagnostics);
 	} else {
 		g_set_error (err, GMIME_ERROR, errno,
-			     _("Failed to %s vua GnuPG: %s\n"),
+			     _("Failed to %s via GnuPG: %s\n"),
 			     mode, g_strerror (errno));
 	}
 	
