@@ -2,20 +2,19 @@
 /*  GMime
  *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public License
- *  as published by the Free Software Foundation; either version 2.1
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  This library is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free
- *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
- *  02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -91,7 +90,6 @@ g_mime_iconv_strndup (iconv_t cd, const char *string, size_t n)
 	char *out, *outbuf;
 	const char *inbuf;
 	size_t outlen;
-	int errnosav;
 	
 	if (cd == (iconv_t) -1)
 		return g_strndup (string, n);
@@ -119,7 +117,7 @@ g_mime_iconv_strndup (iconv_t cd, const char *string, size_t n)
 		 * We just need to grow our outbuffer and try again.
 		 */
 		
-		converted = outbuf - out;
+		converted = outbuf - (char *) out;
 		if (errno == E2BIG) {
 			outlen += inleft * 2 + 16;
 			out = g_realloc (out, outlen + 4);
@@ -152,17 +150,13 @@ g_mime_iconv_strndup (iconv_t cd, const char *string, size_t n)
 	
  fail:
 	
-	errnosav = errno;
-	
-	w(g_warning ("g_mime_iconv_strndup: %s at byte %lu", strerror (errno), n - inleft));
+	w(g_warning ("g_mime_iconv_strndup: %s", strerror (errno)));
 	
 	g_free (out);
 	
 	/* reset the cd */
 	iconv (cd, NULL, NULL, NULL, NULL);
 	
-	errno = errnosav;
-	
 	return NULL;
 }
 
