@@ -1,21 +1,20 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2009 Jeffrey Stedfast
+ *  Copyright (C) 2000-2007 Jeffrey Stedfast
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Lesser General Public License
- *  as published by the Free Software Foundation; either version 2.1
- *  of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
- *  This library is distributed in the hope that it will be useful,
+ *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Lesser General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public
- *  License along with this library; if not, write to the Free
- *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
- *  02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 
@@ -25,21 +24,6 @@
 
 #include "gmime-stream-file.h"
 
-
-/**
- * SECTION: gmime-stream-file
- * @title: GMimeStreamFile
- * @short_description: A Standard-C FILE-based stream
- * @see_also: #GMimeStream
- *
- * A simple #GMimeStream implementation that sits on top of the
- * Standard C FILE pointer based I/O layer. Unlike #GMimeStreamFs, a
- * #GMimeStreamFile will typically buffer read and write operations at
- * the FILE level and so it may be wasteful to wrap one in a
- * #GMimeStreamBuffer stream.
- **/
-
-
 static void g_mime_stream_file_class_init (GMimeStreamFileClass *klass);
 static void g_mime_stream_file_init (GMimeStreamFile *stream, GMimeStreamFileClass *klass);
 static void g_mime_stream_file_finalize (GObject *object);
@@ -73,7 +57,7 @@ g_mime_stream_file_get_type (void)
 			NULL, /* class_finalize */
 			NULL, /* class_data */
 			sizeof (GMimeStreamFile),
-			0,    /* n_preallocs */
+			16,   /* n_preallocs */
 			(GInstanceInitFunc) g_mime_stream_file_init,
 		};
 		
@@ -129,7 +113,7 @@ static ssize_t
 stream_read (GMimeStream *stream, char *buf, size_t len)
 {
 	GMimeStreamFile *fstream = (GMimeStreamFile *) stream;
-	size_t nread;
+	ssize_t nread;
 	
 	if (stream->bound_end != -1 && stream->position >= stream->bound_end)
 		return -1;
@@ -143,14 +127,14 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 	if ((nread = fread (buf, 1, len, fstream->fp)) > 0)
 		stream->position += nread;
 	
-	return (ssize_t) nread;
+	return nread;
 }
 
 static ssize_t
 stream_write (GMimeStream *stream, const char *buf, size_t len)
 {
 	GMimeStreamFile *fstream = (GMimeStreamFile *) stream;
-	size_t nwritten;
+	ssize_t nwritten;
 	
 	if (stream->bound_end != -1 && stream->position >= stream->bound_end)
 		return -1;
@@ -164,7 +148,7 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 	if ((nwritten = fwrite (buf, 1, len, fstream->fp)) > 0)
 		stream->position += nwritten;
 	
-	return (ssize_t) nwritten;
+	return nwritten;
 }
 
 static int
@@ -209,6 +193,7 @@ static int
 stream_reset (GMimeStream *stream)
 {
 	GMimeStreamFile *fstream = (GMimeStreamFile *) stream;
+	int rv;
 	
 	if (fstream->fp == NULL)
 		return -1;
@@ -264,7 +249,7 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 	if (stream->bound_end != -1 && real > stream->bound_end)
 		return -1;
 	
-	if (fseek (fp, real, SEEK_SET) == -1 || (real = ftell (fp)) == -1)
+	if (fseek (fp, real, SEEK_END) == -1 || (real = ftell (fp)) == -1)
 		return -1;
 	
 	stream->position = real;
